Introduction
============

This widget represents the reference implementation of the "Adding preferences"
tutorial. This tutorial is based on previous tutorials were we have used a
hardcoded value of the API key for the weather underground service. This may be
good enough for a basic version of the widget, but in the real world this
would be better implemented as a preference.

For this tutorial, we're are going to use the proposed solution from the
"[Making requests and wiring][prev_tutorial]" tutorial as base. You can download
the code of the widget using this [link][initial_code].

[prev_tutorial]: http://conwet.fi.upm.es/docs/display/wirecloud/Making+requests+and+wiring
[initial_code]: http://conwet.fi.upm.es/docs/download/attachments/4423705/Example2Skel.zip

Adding preferences to the config.xml
------------------------------------

The first step for adding a preference is declaring it in the config.xml file.
This can be accomplished by adding a Preference element into the
Platform.Preferences element. The final result should look like:

    :::xml

    ...
     
        <preferences>
            <preference name="api_key" type="text" description="Weather Underground API key" label="API key" />
        </preferences>
     
    ...
 

* The **name** attribute will be use to reference this preference when using the
  javascript API.
* The **type** attribute indicates the type of data this preference is going to
  store. Valid values are: boolean, list, password and text.
* The **label** attribute will be used, for example, in the user interface to
  ask the user for a value for this preference. Also, this attribute can be
  translated whereas the name attribute not.
* The **description** attribute is used to provided a more description about the
  preference.

This is the same declaration but using RDF (turtle):

    :::n3

    ...
        wire:hasPlatformPreference [ a wire:PlatformPreference ;
                rdfs:label "API key" ;
                dcterms:description "Weather Underground API key" ;
                dcterms:title "api_key" ;
                wire:index "0" ;
                wire:type "text" ] ;
    ...

Basic usage
-----------

Once declared the users preference in the widget descriptor, you will be able to
read the current value of the preference using the
[MashupPlatform.prefs.get][prefs.get] method. For example, taking into account
the code used in previous tutorials:

    :::javascript

    var getForecastByCoord = function getForecastByCoord(coord, onSuccess, onError) {
     
        ...
     
        url = 'http://api.wunderground.com/api/' + API_KEY + '/conditions/forecast/q/';
        url += coord.lat + ',' + coord.lon;
        url += '.json';
     
        MashupPlatform.http.makeRequest(url, {
    
        ...
    };

Should be transformed to:

    :::javascript

    var getForecastByCoord = function getForecastByCoord(coord, onSuccess, onError) {
     
        ...
     
        url = 'http://api.wunderground.com/api/' + MashupPlatform.prefs.get('api_key') + '/conditions/forecast/q/';
        url += coord.lat + ',' + coord.lon;
        url += '.json';
    
        MashupPlatform.http.makeRequest(url, {
    
        ...
    };

After this change, the API_KEY constant is not used anymore, so you can safely remove it from the source code.


[prefs.get]: http://conwet.fi.upm.es/docs/display/wirecloud/Javascript+API#JavascriptAPI-MashupPlatformprefsget

Reacting to preference changes
------------------------------

Some times widgets need to react to changes in the preferences. This can be
accomplished using the
[MashupPlatform.prefs.registerCallback][prefs.registerCallback] method. For
example, we can make use of this support for displaying an alert message to the
user just in case the new value for the api-key preference is not valid:

    :::javascript

    var updateMissingAPIKeyAlert = function updateMissingAPIKeyAlert() {
        if (MashupPlatform.prefs.get('api_key').trim().length !== 16) {
            document.getElementById('api-key-alert').classList.remove('hide');
        } else {
            document.getElementById('api-key-alert').classList.add('hide');
        }
    };
     
    MashupPlatform.prefs.registerCallback(function (new_values) {
        if ('api_key' in new_values) {
           updateMissingAPIKeyAlert();
        }
    });

Evidently, this only check the value of the preference when this is changed, so
you need to call manually the updateMissingAPIKeyAlert function when the widget
is loaded. This can be acomplished using the following code:

    :::javascript

    window.addEventListener('load', function () {
        updateMissingAPIKeyAlert();
    }, true);


[prefs.registerCallback]: http://conwet.fi.upm.es/docs/display/wirecloud/Javascript+API#JavascriptAPI-MashupPlatformprefsregisterCallback 

Testing it
----------


